declare type StateType = 'Task' | 'Map' | 'Pass' | 'Wait' | 'Choice' | 'Succeed' | 'Fail';

interface BaseState {
    Type: StateType;
    Comment?: string;
}

declare type PayloadTemplate = object;
interface CanHaveInputPath {
    InputPath?: string | null;
}
interface CanHaveParameters {
    Parameters?: PayloadTemplate;
}
interface CanHaveResultSelector {
    ResultSelector?: PayloadTemplate;
}
interface CanHaveResultPath {
    ResultPath?: string | null;
}
interface CanHaveOutputPath {
    OutputPath?: string | null;
}

declare type StringOperatorNames = 'StringEquals' | 'StringLessThan' | 'StringGreaterThan' | 'StringLessThanEquals' | 'StringGreaterThanEquals' | 'StringMatches';
declare type StringPathOperatorNames = 'StringEqualsPath' | 'StringLessThanPath' | 'StringGreaterThanPath' | 'StringLessThanEqualsPath' | 'StringGreaterThanEqualsPath';
declare type NumericOperatorNames = 'NumericEquals' | 'NumericLessThan' | 'NumericGreaterThan' | 'NumericLessThanEquals' | 'NumericGreaterThanEquals';
declare type NumericPathOperatorNames = 'NumericEqualsPath' | 'NumericLessThanPath' | 'NumericGreaterThanPath' | 'NumericLessThanEqualsPath' | 'NumericGreaterThanEqualsPath';
declare type BooleanOperatorNames = 'BooleanEquals';
declare type BooleanPathOperatorNames = 'BooleanEqualsPath';
declare type TimestampOperatorNames = 'TimestampEquals' | 'TimestampLessThan' | 'TimestampGreaterThan' | 'TimestampLessThanEquals' | 'TimestampGreaterThanEquals';
declare type TimestampPathOperatorNames = 'TimestampEqualsPath' | 'TimestampLessThanPath' | 'TimestampGreaterThanPath' | 'TimestampLessThanEqualsPath' | 'TimestampGreaterThanEqualsPath';
declare type TypeTestOperatorNames = 'IsNull' | 'IsPresent' | 'IsNumeric' | 'IsString' | 'IsBoolean' | 'IsTimestamp';
declare type StringComparisonOperator = {
    [P in StringOperatorNames]?: string;
};
declare type StringPathComparisonOperator = {
    [P in StringPathOperatorNames]?: string;
};
declare type NumericComparisonOperator = {
    [P in NumericOperatorNames]?: number;
};
declare type NumericPathComparisonOperator = {
    [P in NumericPathOperatorNames]?: string;
};
declare type BooleanComparisonOperator = {
    [P in BooleanOperatorNames]?: boolean;
};
declare type BooleanPathComparisonOperator = {
    [P in BooleanPathOperatorNames]?: string;
};
declare type TimestampComparisonOperator = {
    [P in TimestampOperatorNames]?: string;
};
declare type TimestampPathComparisonOperator = {
    [P in TimestampPathOperatorNames]?: string;
};
declare type TypeTestComparisonOperator = {
    [P in TypeTestOperatorNames]?: true;
};
declare type ComparisonOperator = StringComparisonOperator | StringPathComparisonOperator | NumericComparisonOperator | NumericPathComparisonOperator | BooleanComparisonOperator | BooleanPathComparisonOperator | TimestampComparisonOperator | TimestampPathComparisonOperator | TypeTestComparisonOperator;
declare type BaseDataTestExpression = {
    Variable: string;
};
declare type DataTestExpression = BaseDataTestExpression & ComparisonOperator;
declare type BooleanExpression = {
    And?: ChoiceRuleWithoutNext[];
    Or?: ChoiceRuleWithoutNext[];
    Not?: ChoiceRuleWithoutNext;
};
declare type ChoiceRuleWithoutNext = DataTestExpression | BooleanExpression;
declare type ChoiceRule = (DataTestExpression | BooleanExpression) & {
    Next: string;
};
interface BaseChoiceState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Choice';
    Choices: ChoiceRule[];
    Default?: string;
}
declare type ChoiceState = BaseChoiceState;

interface EndableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    End: true;
}
interface SucceedOrFailState extends BaseState {
    Type: Extract<StateType, 'Succeed' | 'Fail'>;
}
declare type TerminalState = EndableState | SucceedOrFailState;

interface BaseFailState extends BaseState {
    Type: 'Fail';
    Cause?: string;
    Error?: string;
}
declare type FailState = TerminalState & BaseFailState;

interface NextableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    Next: string;
}
declare type IntermediateState = NextableState;

interface BaseMapState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Map';
    Iterator: Omit<StateMachineDefinition, 'Version' | 'TimeoutSeconds'>;
    ItemsPath?: string;
    MaxConcurrency?: number;
}
declare type MapState = (IntermediateState | TerminalState) & BaseMapState;

declare type JSONValue = null | boolean | number | string | object | JSONValue[];

interface BasePassState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Pass';
    Result?: JSONValue;
}
declare type PassState = (IntermediateState | TerminalState) & BasePassState;

interface BaseSucceedState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Succeed';
}
declare type SucceedState = TerminalState & BaseSucceedState;

interface BaseTaskState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Task';
    Resource: string;
}
declare type TaskState = (IntermediateState | TerminalState) & BaseTaskState;

interface BaseWaitState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Wait';
    Seconds?: number;
    Timestamp?: string;
    SecondsPath?: string;
    TimestampPath?: string;
}
declare type WaitState = (IntermediateState | TerminalState) & BaseWaitState;

declare type AllStates = TaskState | MapState | PassState | WaitState | ChoiceState | SucceedState | FailState;

interface StateMachineDefinition {
    States: Record<string, AllStates>;
    StartAt: string;
    Comment?: string;
    Version?: string;
    TimeoutSeconds?: number;
}

declare type TaskStateResourceLocalHandler = {
    [taskStateName: string]: (input: JSONValue) => Promise<JSONValue>;
};
declare type WaitStateTimeOverride = {
    [waitStateName: string]: number;
};
interface Overrides {
    taskResourceLocalHandlers?: TaskStateResourceLocalHandler;
    waitTimeOverrides?: WaitStateTimeOverride;
}
interface RunOptions {
    overrides?: Overrides;
    noThrowOnAbort?: boolean;
}
interface ValidationOptions {
    readonly checkPaths?: boolean;
    readonly checkArn?: boolean;
}

declare class StateMachine {
    /**
     * The structure of the State Machine as represented by the Amazon States Language.
     */
    private readonly definition;
    /**
     * A map of functions to execute each type of state.
     */
    private readonly stateExecutors;
    /**
     * Options to control whether to apply certain validations to the state machine definition.
     */
    private readonly validationOptions;
    /**
     * Constructs a new state machine.
     * @param definition The state machine definition defined using the Amazon States Language (https://states-language.net/spec.html).
     * @param validationOptions Options to control whether to apply certain validations to the definition.
     * These options also apply to state machines defined in  the `Iterator` field of `Map` states.
     */
    constructor(definition: StateMachineDefinition, validationOptions?: ValidationOptions);
    /**
     * Executes the state machine, running through the states specified in the definition.
     *
     * By default, if the execution is aborted, the result will throw an `ExecutionAbortedError`. This behavior can be changed by setting
     * the `noThrowOnAbort` option to `true`, in which case the result will be `null`.
     *
     * @param input The input to pass to this state machine execution.
     * @param options Miscellaneous options to control certain behaviors of the execution.
     */
    run(input: JSONValue, options?: RunOptions): {
        abort: () => void;
        result: Promise<JSONValue>;
    };
    /**
     * Helper method that handles the execution of the machine states and the transitions between them.
     */
    private execute;
    /**
     * Process the current input according to the `InputPath` and `Parameters` fields.
     */
    private processInput;
    /**
     * Process the current result according to the `ResultSelector`, `ResultPath` and `OutputPath` fields.
     */
    private processResult;
    /**
     * Handler for task states.
     *
     * Invokes the Lambda function specified in the `Resource` field
     * and sets the current result of the state machine to the value returned by the Lambda.
     */
    private executeTaskState;
    /**
     * Handler for map states.
     *
     * Iterates over the current input items or the items of an array specified
     * by the `ItemsPath` field, and then processes each item by passing it
     * as the input to the state machine specified in the `Iterator` field.
     */
    private executeMapState;
    /**
     * Handler for pass states.
     *
     * If the `Result` field is specified, copies `Result` into the current result.
     * Else, copies the current input into the current result.
     */
    private executePassState;
    /**
     * Handler for wait states.
     *
     * Pauses the state machine execution for a certain amount of time
     * based on one of the `Seconds`, `Timestamp`, `SecondsPath` or `TimestampPath` fields.
     */
    private executeWaitState;
    /**
     * Handler for choice states.
     *
     * Evaluates each choice rule specified in the `Choices` field.
     *
     * If one of the rules matches, then the state machine transitions to the
     * state specified in the `Next` field for that choice rule.
     *
     * If no rule matches but the `Default` field is specified,
     * then the next state will be the state specified in said field.
     *
     * If no rule matches and the `Default` field is not specified, throws a
     * States.NoChoiceMatched error.
     */
    private executeChoiceState;
    /**
     * Handler for succeed states.
     *
     * Ends the state machine execution successfully.
     */
    private executeSucceedState;
    /**
     * Handler for fail states.
     *
     * Ends the state machine execution and marks it as a failure.
     */
    private executeFailState;
}

declare abstract class CustomError extends Error {
    constructor(message?: string);
    abstract toString(): string;
}

/**
 * Represents the abortion of a state machine execution.
 */
declare class ExecutionAbortedError extends CustomError {
    constructor();
    toString(): string;
}

export { ExecutionAbortedError, StateMachine };
